@@ -4,7 +4,8 @@
 
 import sys, json
 
-sys.path.insert(0, '..')
+# remove before flight
+#sys.path.insert(0, '..')
 
 from STEP.REST import *
 
